@@ -53,7 +53,7 @@ setMethod("toString", signature=c("unknown_statement"), definition=function(obje
     } else {
       retValue <- object@line
     }
-  } else if (dest=="RxODE" || dest=="mrgsolve" || dest=="NONMEM") {
+  } else if (isRxODE(dest) || dest=="mrgsolve" || dest=="NONMEM") {
     retValue <- object@line
   } else {
     UnsupportedDestException()
